#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"column.hpp"
#include	"fileio.hpp"

#define	BUFFER_SIZE		4096
#define	EOF_CHAR			0x1a

/* ---------------------------------------------------------------------
	Set up the input and output files with their buffers.
   --------------------------------------------------------------------- */

void	Fileio::buffers(char *pszFilename, BOOL fRead, int iRecLen, BOOL fTextMode)
{
	pchBuffer = 0;
	pchRecord = 0;

	if ((hFile = fopen(pszFilename, fRead ? "rb" : "wb")) == 0)
		error_handler(1, pszFilename);

	if ((pchBuffer = (char*) new char[BUFFER_SIZE]) == 0)
		error_handler(4, pszFilename);

	if (setvbuf(hFile, pchBuffer, _IOFBF, BUFFER_SIZE) != 0)
		error_handler(2, pszFilename);

	this->iRecLen = iRecLen;

	if ((pchRecord = (char*) new char[iRecLen]) == 0)
		error_handler(4, pszFilename);

	this->fTextMode = fTextMode;
}

/* ---------------------------------------------------------------------
	Read a single record as fixed length or variable delimited with CR-LF.
	Return value is FALSE if end-of-file encountered.
   --------------------------------------------------------------------- */

BOOL	Fileio::read_record()
{
	int	i;

	if (feof(hFile))
		return FALSE;

	if (fTextMode) {
		if (0 == fgets(pchRecord, iRecLen, hFile)) {
			if (feof(hFile))
				return FALSE;

			error_handler(10,"input file");
		}

		/* skip saving of CR & LF chars */
		i = strlen(pchRecord);
		if (i > 0 && pchRecord[i-1] == '\n') i--;
		if (i > 0 && pchRecord[i-1] == '\r') i--;

		memset(&pchRecord[i], 0, iRecLen - i);
	}
	else {
		i = fread(pchBuffer, sizeof(char), iRecLen, hFile);
		if (i != iRecLen) {
			if (i == 0 && feof(hFile))
				return FALSE;

			if (i == 1 && feof(hFile) && pchBuffer[0] == EOF_CHAR)
				return FALSE;

			error_handler(10,"input file");
		}
	}
	return TRUE;
}

/* ---------------------------------------------------------------------
	Write a single record to a file.
	The arguments are:
		pFile			=	output file
		pchOutput	=	buffer containing record
		reclen		=	maximum / actual record length
	No return value.
   --------------------------------------------------------------------- */

void	Fileio::write_record(const char *pchOutput, int iRecLen)
{
	int	i, cch;

	if (fTextMode) {
		for (i = 0, cch = 0; (i < iRecLen-1) && (pchOutput[i] != 0); i++)
			cch++;

		if (cch > 0 && fwrite(pchOutput, cch, 1, hFile) != 1)
			error_handler(10,"output file");

		fputs("\r\n", hFile);
	}
	else {
		if (fwrite(pchOutput, iRecLen, 1, hFile) != 1)
			error_handler(10,"output file");
	}
}

/* ---------------------------------------------------------------------
	Free any dynamically allocated storage and close all open files.
   --------------------------------------------------------------------- */

void	Fileio::Cleanup()
{
	if (hFile != 0)
		if (fclose(hFile) != 0)
			error_handler(3, 0);

	delete[] pchBuffer;
	delete[] pchRecord;
}


